# Table of Contents #

- [Overview](#overview)
- [Combat Rules](#combat-rules)
- [Status](#status)
- [Secondary Statistics](#statistics)
- [Combat Implementation](#combat-implementation)
- [Combat State](#combat-state)
- [Combat UI](#combat-ui)
    * [Board](#board)
    * [Log](#log)
    * [Commands](#commands)
    * [Combat pages](#combat-pages)

# Overview #

Combat will likely be the most complex part of the game, and will require
a very careful design. Our requirements are the following:

1. Combat rules should be simple.
2. Combat rules should provide several degrees of freedom.
3. It should be easy to craft challenging AI opponents that aren't complete
    idiots.
4. It should be easy to craft characters with a wide range of different powers
    highly conducive to synergy.
5. The skills you need to be successful in combat are NOT luck, spatial
    awareness or fast reflexes. Instead, it should about deciding which powers
    should be used when.

When we say that combat rules should provide several degrees of freedom, what we
mean is that combat should provide enough dials to let the player do interesting
things and have interesting emergent properties. For example, the player should
be able to manipulate statuses, weaken them, add them, remove them, move them
around, that kind of thing.

If we can satisfy requirements 1 and 2, then requirements 3 and
4 should fall out in the wash.

# Combat Rules #

The rules of combat need to be crafted to satisfy the following requirements:

1. Combat is challenging. A very careful use of powers will be needed to defeat
    enemies.
2. Combat should have as little nondeterminism as possible. Combat should be
    a puzzle as much as anything.
3. Combat should be fast-paced.
4. Combat should be dynamic. When we say dynamic we have two meanings:
    a. Each battle should pose a mostly unique challenge
    b. It should feel like combatants are in fact moving all over the
        battlefield rather than standing around taking turns punching each other.
        Combat should feel more like [this](https://www.youtube.com/watch?v=bPXaKQr_VW0)
        and less like [this](https://www.youtube.com/watch?v=hfSz0bQzZuM)

The rules are as follows:

1. Each character has a collection of actions they can take, and the number of
times each round they can take that action. For example, Scarlet Moon can
attack once an use a power once, while Anklyana can use a power twice, but 
can't attack. The character can use any combination of these actions in
any order. For example, Scarlet Moon can:

a. Attack and use a power.
b. Use a power and attack.
c. Attack.
d Use a power.
e. Do nothing.

2. If you want to modify a character, and have the modification expire
eventually, you have to do it by applying a status. Statuses can modify
any of the character's secondary statistics, and can do damage, but that's
it (they can't redirect damage, trigger a reaction from a character, etc).
These are basically number manipulators.

3. Other effects can modify secondary statistics. For example, scarlet fists
transfers some of Scarlet Moon's power stat into her strength and speed, but 
never expires. It has be explicitly canceled by the player. These are not
considered statuses, and are not affected by things that manipulate statuses.

4. Powers can do just about anything. They can apply a status, do damage,
modify a character's secondary statistics, move statuses around, etc. Most
of them will do damage or apply a status. Some powers (like scarlet fists)
may modify a character's statistics directly, without applying a status.

# Status #

A status is a temporary modifier on a character. It may provide stat
bonuses or penalties, restrict or enhance certain types of powers, or what
have you.

A status is a table with four keys:
    `apply` - An operation on the target character, combatstate, user,
        and a number representing the potency of the status.
        It applies the status to the specified
        character for the specified duration. This is invoked when the target
        is not already afflicted with this status.
    `stack` - An operation with the same arguments as `apply`. This is
        invoked when the target is already inflicted with this status. Note that
        the engine automatically adds the new duration to the current duration, so
        this function only needs to exist if stacking this status does more than
        increase its duration.
    `remove` - An operation on the target character, and combatstate. This is invoked when
    `startturn` - An optional operation on target, and combatstate,
        that is executed at the start of the target's turn. This can do anything (like
        do damage, cancel another status, etc).
    `endturn` - The end of turn equivalent of `startturn` that gets executed when
    the character's turn ends.
    `name` - The name of the status
    `display` - A (very short) string that will be displayed next to a character's
        name to represent the status in the combat menu. If `nil`, defaults
        to the first character of the status' name.
    `concisename` - A short (usually only a few letters) name for the status,
        used in the concise view of the combat board.
    `buff` - Either a number, or a function that takes the character inflicted
        by the status and returns a number providing a rough idea of how beneficial the
        status is. A positive number means the status is beneficial, a negative number
        means the staus is detrimental.  The more positive, the better the buff. The
        more negative, the worse the debuff.  Note that these numbers are arbitrary, and
        are left up to the developer to define.


Statuses should be defined in `data/statuses.lua`

Statuses are stored in a table `character.statuses` that only exists during combat.
`character.statuses` is a list of tables with two keys:
`status` containing the table defined above, and `duration` containing the number of
rounds the status should last.

# Secondary Statistics #

The game already has several primary statistics: strength, speed, power,
energy, maxenergy, spanker, spankee, victory, defeat. Of these, only strength,
speed, power, energy and maxenergy, have any impact on combat.

For combat, we also have a collection of secondary statistics that 
can be twiddled in addition to the primary statistcs. These are all
stored in the table `character.secondarystatistics` during combat, 
and don't exist outside of combat.

1. Attack - The maximum damage a character can do. By default, it's
strength + speed/2. However, statuses can apply modifiers to it. Attack can
never drop below 0. 

Internally, we don't actually store attack directly. Instead, each character has
a table `attackmodifiers` that maps identifiers (typically
status names) to integers, which tracks all the modifiers that should be applied to
the base value of strength + speed/2. Attack is calculated by:

```
   local attack = character.statistics.strength + math.floor(character.statistics.speed/2) 
   for _, modifier in pairs(character.secondarystatistics.attackmodifieres) do
        attack = math.max(0, finalattack + modifier)
   end
   return attack
```

By storing a table of named modifiers, it's very easy for statuses to apply and remove
modifiers. Every modification should have a name, even if it's "permanent" (i.e. not
applied through a status). We just don't care what the name is in that case.

2. Defense - How much the character's innate defenses reduce the damage they take.
By default, it's speed. Like with attack, statuses can apply modifiers to it, and it
never drops below 0.

Defense is stored in the same manner as attack, except the table is called `defensemodifiers`
instead of `attackmodifiers`.

A target's defense is subtracted from an attacker's attack to get the final damage dealt:
```
local damage = math.max(0, attack(attacker) - defense(target))
```

3. Shield - Temporary hitpoints basically. When a character is damaged, the damage comes
out of shield before it comes out of energy. It never drops below 0 (at which point, it does
nothing). Like attack and defensemodifier, this is a table that maps a name (typically a
status name) to the bonus (or penalty!) to the character's shield.

4. Retaliation - A stored pool of damage. Retaliate works like shield, except instead of
blocking damage, it does retaliatory damage againsta target. For example, suppose 
Scarlet Moon has retaliation 25, and Buzzsaw hits her for 50. Then, Scarlt Moon takes
50 damage, and retaliates for 25, draining the retaliation pool. Now, suppose instead
Buzzsaw hit Scarlet Moon for 20 damage, and then again for 20 damage. The first time,
Scarlet Moon retaliates for 20 damage, draining her pool to 5. The second time, she
retaliates for 5 damage (since that's all that's left in the pool), dropping it to 0.
Like with shield, at the end of the round all retaliation pools greater than 0 drop
by 1. Note that retaliation ignores the target's defense, but does *not* ignore 
the target's shield. Furthermore, retaliation is computed after the damage has
been reduced by shield, and retaliation does not trigger retaliation.

So once damage has been computed, we reduce a character's energy as follows:
```
local damage = computedamage(attacker, target)
local secondarystatistics = target.secondarystatistics
local shield = target.secondarystatistics.shield
secondarystatistics.shield = math.max(0, shield - damage)
damage = math.max(0, damage - shield)
local retaliatorydamage = math.max(target.secondarystatistics.retaliation, damage)
target.secondarystatistics.retaliation = math.max(0, retaliation - damage)
target.statistics.energy = target.statistics.energy - damage
shield = user.statistics.shield 
target.statistics.shield = shield - retaliatorydamage
retaliatorydamage = retaliatorydamage - shield
user.statistics.energy = user.statistics.energy - retaliatorydamage
```

5. Attrition - How much energy the character loses at the end of the round.
Note that attrition can never drop a character below 1 energy.


# Combat Implementation #

At the start of each round, we invoke the function `combat/combat.choices`,
which takes the `combatstate` and returns a list of choices that need to
be made, in the order they should be made. The default implementation asks
for all player characters first, followed by all enemies.

We then iterate through each choice and resolve them into actions. For players,
this consists of selecting an action, and resolving it.

For the AI, it consists of using the chooser's AI (see [AI](./ai.md)), to map
choices to actions.

Then we end the round, incrementing the `combstate.round` counter.

# Combat State #

The `combatstate` is the one source of truth for all data about combat. It is
a table with the following keys:

1. `start` - A table that defines the starting state of combat. It
    contains the following keys:
        a. `allies` - The original list of allies involved
        b. `enemies` - The original list of enemies involved
3. `round` - A number representing the current round, starting at 1.
4. `choice` - The current choice to be resolved.
6. `victory` - The page to transition to if the player is victorious.
7. `loss` - The page to transition to if the player is defeated. If `nil`, then
    the player must win to advance in the game.
8. `allies` - A list of the characters controlled by the player
9. `enemies` - A list of the characters controlled by the enemy
10. `characters` - The concatenation of `allies` and `enemies`
11. `charindex` - The index of the currently active character in
    `combatstate.characters`
11. `combatlog` - A list of paragraphs containing previous messages from combat
12. `metalog` - A list of paragraphs containing data that should be displayed
    along the right side of the screen, like the name of the
    currently active character, their current options, information about any
    selected characters, or powers etc.
12. `modifiers` - A list of all active modifier primitives
13. `selections` - An optional field that contains the list of selection keys
    that have been pressed by the player. All numeric and function keys are
    selection keys.
14. `choicecounts` - A table that maps character name to an ordered table of
    actions keyed on name. Each action is a table with three keys:
        a. `max` - The maximum number of times the named character can
            perform this action each round.
        b. `current` - The remaining number of times the named character
            can perform this action each round.
        c. `page` - A combat page for this action
15. `attrition` - A function that takes a character and the current state of
    combat and returns the amount of energy that character should lose to
    attrition at the end of the round. If `nil`, then attrition is not applied.
17. `ai` - A string describing the AI that is currently being used (i.e.
    "dumb", "average", or "smart")
18. `nextround` - Whether or not we are transitioning the nextround. This is a
    bit of a hack, because we're using the backup functionality for the uistate,
    but the logic that actually determines if we're moving onto the next round
    happens at the combatstate level (where we know nothing about the uistate).
    `nextround` is a flag to notify the UI logic that we've transitioned to a
    new round and should clear the undo. It is flipped back to false by the
    UI code after the undo has been cleared.
19. `currentpage` - The combatpage we're currently executing. This is another
    hack to make backup work. Before, I was similar building the current page,
    but this meant that we weren't properly preserving the player's chain of
    choices. For example, if you triggered a power, but then backed up that
    power, it wouldn't display the power selection screen like it's supposed to.
    However, by having the combat state remembering its current page, when we
    restore the backup we can just execute that backed up state's page.

# Combat UI #

The UI is broken into three main regions: the board, the log, and the choices:

------------------------
|   Enemies            |
|----------------------|
|   Allies             |
------------------------
|                      |
|   Log                |
|                      |
------------------------
|   Choices            |
------------------------

The log has the largest allocation of screen. The buttons have the smallest.

## Board ##

The board has two lists of characters: The enemies involved in combat, and
the allies.

Each character has a status string with the following format:

(Key)ID:current energy/maximum energy:Status1(Duration1);Status2(Duration2);...

(1)D:35/35:P(1)
------------------
2 (F1)H:35/35
------------------

In this example, we have one enemy, and one ally: `D` is an enemy who can be
selected with
the number keys. `H` is an ally who can be selected with a function key. `D`
is afflicted with the paralyzed status. `D`'s status will wear off next
round. `D` and `H` both have their
maximum energy (35).

## Log ##

The log is simply a scrollable log of text. It includes information about
choices a player may choose, as well as the results of each action:

-----------------
|              /|\
|  Text!        |
|              \|/
-----------------


## Choices ##

The choices are the list of choices available to the player. Each choice
has a name, with a single character wrapped in parenthesis. This character
is the key the user presses to use the action:

----------------------------------
| (A)ttack:1    (<==)Back        |
| (P)ower: 1    (M)ove: 1        |
| (R)estart Combat (T)itle Screen|
----------------------------------

## Combat Page ##

Developers define the combat through a combat page. A combat page is a standard
page on combatstate.
