--[[
-- Copyright 2017 AKA Russell
--
-- This file is part of Scarlet.
--
-- Scarlet is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- Scarlet is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with Scarlet.  If not, see <http://www.gnu.org/licenses/>.
--]]

--[[
-- Defines the power choice. There are two components: Selecting a power,
-- and walking the decision tree for that power. We select a power using the
-- same basic infrastructure as when we select targets, except we use
-- the (English) alphabet rather than numbers.
--
-- We start walking the decision tree by having the power selection choice
-- map to a page that determines the selected power, and then mirrors that
-- power's page functions.
-- ]]

local combatstateops = require 'combat/combatstate'
local scarlettables = require 'scarlettables'
local pages = require 'pages'
local log = require 'log'

local M = {}

local alphabet = {'a', 'b', 'c', 'd', 'e', 'f', 'g', 'h', 'i', 'j', 'k', 'l',
    'm', 'n', 'o', 'p', 'q', 'r', 's', 't',  'u', 'v', 'w', 'x', 'y', 'z'}

M.HEALS = -1
M.NEITHER = 0
M.DAMAGES = 1

M.ALL_TARGETS = -1
M.ALL_OPPONENTS = -2
M.ALL_ALLIES = -3

-- next char, load, title, restart
local reservedcharacters = {e=1, l=1, t=1, r=1}

local vchars = {}
for _, character in ipairs(alphabet) do
    if not reservedcharacters[character] then
        table.insert(vchars, character)
    end
end

local function powertransformer(combatstate)
    local activecharacter = combatstate.characters[combatstate.charindex]
    combatstate.metalog = {}
    local metalog = combatstate.metalog
    local lineofpowers = {}
    for index, powername in ipairs(activecharacter.powers._keys or {}) do
        table.insert(lineofpowers, '(' .. vchars[index] .. ')' .. powername)
        if index % 3 == 0 then
            table.insert(metalog, table.concat(lineofpowers, '\t'))
            lineofpowers = {}
        end
    end
    if next(lineofpowers) then
        table.insert(metalog, table.concat(lineofpowers, ''))
    end
end

local function powerchoices()
    local powertext = "(@)Select a Power"
    return {
        [powertext]=M.selectedpower,
        _keys={powertext}
    }
end

M.power = pages.page(nil, nil, powerchoices, powertransformer)

local function selectedpowertransformer(combatstate)
    local activecharacter = combatstate.characters[combatstate.charindex]
    local selectionindex = scarlettables.reverseLookup(
        combatstate.selections[1],
        vchars
    )
    combatstate.selections = {}
    local powername = activecharacter.powers._keys[selectionindex]
    if powername then
        local power = activecharacter.powers[powername]
        combatstate.selectedpower = power
        power.transformer(combatstate)
    else
        combatstate.selectedpower = nil
    end
end

local function selectedpowerbody(combatstate)
    if combatstate.selectedpower then
        local activecharacter = combatstateops.activecharacter(combatstate)
        local power = combatstate.selectedpower
        local description = power.description or ""
        if type(description) == "function" then
            description = description(activecharacter)
        end
        local body = {description}
        local powerbody = combatstate.selectedpower.body(combatstate)
        for i, paragraph in ipairs(powerbody) do
            body[i] = paragraph
        end
        return body
    end
    return {}
end

local function selectedpowerchoices(combatstate)
    return combatstate.selectedpower
        and combatstate.selectedpower.choices(combatstate)
        or {}
end

local function selectedpowerautomatics(combatstate)
    if combatstate.selectedpower then
        return combatstate.selectedpower.automatics(combatstate)
    end
    return M.power
end

M.selectedpower = pages.page(
    nil,
    selectedpowerautomatics,
    selectedpowerchoices,
    selectedpowertransformer
)

-- [[
-- An operation that applies the cost of a power to a character's energy.
--
-- character: The character to apply the cost of the power to
-- power: The power whose cost should be applied
-- ]]
function M.applycost(character, power)
    local cost = type(power.cost) == "number"
        and power.cost
        or power.cost(character)
    local energy = character.statistics.energy
    character.statistics.energy = math.max(0, energy - cost)
end

return M
