--[[
-- Copyright 2016 AKA Russell
--
-- This file is part of Scarlet.
--
-- Scarlet is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- Scarlet is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with Scarlet.  If not, see <http://www.gnu.org/licenses/>.
--]]
--
--[[
-- This module contains a list of components that define a character (i.e.
-- weapons, armor, any appearance customizations), along with the possible
-- values these components can take.
--]]

local M = {}
local items  = require 'data/items'
M.components = {}

local function addcomponent(component)
    table.insert(M.components, component)
end


--[[
-- Returns a list of all the components that define a character.
--
-- A component is a table with the following _keys:
--  id - The id to use for the GUI widgit that displays this component in the
--      character creation/status screen. This is also used as the key into
--      the character table when accessing that character's value for this
--      component
--  label - The label to display to the player in the character creation/status
--      screen
--  values - A list of the possible values that this component can take. For
--      now this is a list of Strings (i.e. the component values can influence
--      game text, but don't provide any gameplay value). At some point, we'll
--      probably want to make this a table, though that will require some
--      changes to the character creation screen
--  shortcut - The keyboard shortcut to use to select this component in the
--      character creation screen
--]]
function M.buildcomponents()
    return M.components
end

--[[
-- Returns a list of clothing names. By default, the
-- player starts equipped with the first single-slot item to occupy a given
-- slot. If there is no such item for a slot, an error is thrown.
--
-- Note that the name widgit uses F1, while the clothing uses F2-Fn+1, where
-- n is the number of equipment slots available.
-- ]]
function M.clothing()
    local clothingnames = {}

    local defaultclothingnames = {"t-shirt", "jeans", "thong"}
    local defaultclothing = {["t-shirt"]=1, jeans=1, thong=1}
    for _, name in ipairs(defaultclothingnames) do
        table.insert(clothingnames, name)
    end
    for _, clothing in pairs(items) do
        local name = clothing.name
        if not defaultclothing[name] then
            table.insert(clothingnames, name)
        end
    end
    return clothingnames
end

addcomponent {
    id="gender",
    label="Gender: ",
    values= {"female", "male"},
    shortcut="f5"
}

addcomponent {
    id="hairlength",
    label="Hair Length: ",
    values={"short", "shoulderlength", "backlength"},
    shortcut="f6"
}

addcomponent {
    id="hairstyle",
    label="Hair Style: ",
    values={"down", "pig tails", "ponytail", "french braid", "bun", "mohawk", "crown", "single-tail braid"},
    shortcut="f7"
}

addcomponent {
    id="haircolor",
    label="Hair Color: ",
    values={"red", "blonde", "brown", "black", "rainbow"},
    shortcut="f8"
}

addcomponent {
    id="skincolor",
    label="Skin Color: ",
    values={"ivory", "peach", "tanned", "brown", "black"},
    shortcut="f9"
}

addcomponent {
    id="height",
    label="Height: ",
    values={"short", "average", "tall", "gargantuan"},
    shortcut="f10"
}

addcomponent {
    id="musculature",
    label="Musculature: ",
    values={"soft", "fit", "muscular"},
    shortcut="f11"
}

addcomponent {
    id="bodytype",
    label="Body Type: ",
    values={"slim", "slender", "voluptuous", "heavyset"},
    shortcut="f12"
}

return M
