e
# Table of Contents #

- [Overview](#overview)
- [Vocabulary](#vocabulary)
- [Core Concepts](#core-concepts)
    - [State](#state)
        -[Terminology](#terminology)
        -[Canonical Mappings](#canonical-mappings)
        -[Configuration](#configuration)
        - [Game State](#game-state)
        - [UI State](#ui-state)
    - [Page](#page)
        - [Body](#body)
        - [Automatics](#automatics)
        - [Choices](#choices)
        - [Transformer](#transformer)

# Overview #

Pages are a key component of the architecture. A page represents a single
interaction between the user and the system. It has something (usually some sort
of formatted text) that it displays to the player, and it has a collection of
options that the player can select from. UI pages control how the
content and choices are displayed to the player, while backend pages control
the actual content and semantic meaning of each choice.

A key feature of pages is that they are fully self-contained. Each UI
page has complete control over how it and its options are displayed. Backend
pages have complete control over the content they display, and the options
they expose to the player.

# Vocabulary #

- **Ordered Table** - When we refer to an ordered table, we refer to a
    table that also contains a key `keys` that maps to a list of a subset
    of the keys in the table. This list defines the order over which those
    keys should be iterated.
- **Operation** - A sub-routine that returns `nil` and performs side-effects
    on at least one of its arguments.
- **Function** - A sub-routine that may return something other than `nil` and
    is side-effect free.
- **Decision page** - The last page at which the player made a conscious
    decision.

# Core Concepts #

## State ##

The state is our bundle of game data that tracks the player's progress
through the game. If you want to track some sort of data, it goes into one
of the three layers of state:

1. Configuration State

2. Game State

3. UI State

## Terminology ##

### Canonical Mapping ###

The canonical mappings are the mappings defined below. These define the kernel
of each state. These mappings _must_ exist, and they must satisfy the contracts
laid out below. Every component that is aware of each state should be assumed
to be aware of these mappings. These mappings should _not_ be modified except
in very controlled circumstances (i.e. as a part of the `transform` function
in a page).

Note that components should feel free to augment a given state with additional
mappings if they need to save information between iterations. Such mappings
should be considered transient, and should only be used in a very small
segment of the codebase to fulfill a single specific purpose (i.e. preserving
the state of the current UI across iterations). As a rule of thumb, only the
mappings laid out below should be depended on across modules.


### Configuration ###

This state is a table of all the data that is specified when building the
game, and is _immutable_. This includes base character definitions, page
definition and item definitions.

#### Canonical Mappings ####

- `characters` - A table of character definitions keyed on name.
    See the [Characters](./characters.md) file for more detail.
- `items` - A table of items keyed on their names. See
    [Clothing section of the characters](characters/characters.md#clothing) file
    for more detail.
- `stories` - A table of [story pages](./pages.md#story-pages) keyed on name.
- `powers` - A table of [Powers](./powers.md) keyed on name.
- `music` - A table of all the music in the game keyed on name.
- `options` - An ordered table of the configured options, keyed on name. See
    [Options](./utilities/options.md).
- `predicates` - A table of predicates available for use in automatics
- `dynamics` - A table of functions of dynamic text available for use in
    the `body` and `choices` fields of events
- `operations` - A table of operations on `gamestate` available for use in
    the `do` block of game events
- `statuses` - A table of all configured statuses, keyed on name. See
    [status](#combat.md#status) for more details about statuses.

### Game State ###

Game state wraps the configuration,  and is specific to a particular game. It
is _mutable_. It is modified, saved, and loaded by the player. This contains
game-specific state for each character, such as gender, statistics and
equipment.

#### Canonical Mappings #####
- `configuration` - The configuration state.
- `chosenPath` - A number representing the last path chosen by the player (.e.
    choice 1, choice 2, etc).
- `keywords` - A set of values (strings typically) used to track important
    decisions made by the player.
- `options` - A table that maps option names to strings containing the name
    of the selected option
- `characters` - A table of the characters for this playthrough. See
    [Characters](.characters/characters.md). Note that this starts as a deep
    copy of the `characters` table in the `configuration`, and is the table
    modified by in-game pages, while `configuration.characters` remains
    untouched.
- `storypagename` - The name of the current story page
- `lossNumber`- The number of times the player has lost a battle.
- `music` - The key of the song currently being played
- `gametext` - A list of all the paragraphs that have been displayed to the user
    since the last manual choice.
- `visited` - A set of names of events the player has visited in this game.
- `originaloutfit` - An optional mapping that maps slots to clothing. This is
    the outfit the PC was wearing before they changed into their superhero costume.
    It allows them to change back into their original outfit when superheroing
    is all done.
- `combatstate` - The current state of combat (see [Combat](combat/combat.md)).
    This is `nil` outside of combat.


### UI State ###

UI state wraps the game state, and tracks any information associated with the
UI. It is intended that the UI state is held and populated by bridge code
between the UI framework being used (i.e.
[SUIT](http://suit.readthedocs.io/en/latest/index.html) or
standard in/out) and Scarlet. The bridge code will be responsible for
building the initial state UI and passing along pages from the framework
to generic logic that will invoke the appropriate commands in the `UI` state.
It is also responsible for taking the data to display that was generated by
Scarlet and putting it in the appropriate places for the framework to display.

#### Structure #####

- `gui` - The table containing the SUIT functions and state. Manipulate this
    to manipulate the GUI. DO NOT include the suit module directly. Only
    `main.lua` should pull in the `suit` module. Everyone else should access
    the module through the UI state. For one, this removes one obstacle to
    testing the UI code (we can provide our own dummy table for the `suit`
    table). For another, a core philosophy of this codebase is that _all_
    mutable state must be passed in to the operation mutating it. The SUIT
    table is by its very nature mutable, and indeed much of the UI code is
    spent mutating the SUIT table.

- `command` - A string containing the last command provided by the user.

- `core` - A table that contains all of the UI state that is safe to perform
a deep copy of (i.e. the `gui` cannot really be copied and replaced, at least
not the canonical toolkit `SUIT`). The only thing currently stored in `core` is
the `gamestate`

- `save` - A table that tracks metadata about saving:
    - `oldsave` - The index of the currently selected save file
    - `newsave` - The index of the save file the user has asked to select
    - `allsaves` - A list of all the saves that currently exist
    - `textinput` - A table containing a single key: `text` which tracks
        the string to display in the save/load screen's input widgit


Other, temporary, values may be stored at the top level of the UI state
to carry over between frames any data that the UI needs to build itself.
However, this data will _not_ be affected by the back page (which only copies
and restores `uistate.core`). Furthermore, it is highly recommended that these
temporary values do _not_ span more than one GUI, and that they are cleared
when leaving that GUI to protect against state leakage.

##### UI Core Structure #####

- `gamestate` - The [Game State](#game-state).
- `previous` - A deep copy of the previous UI state. If `nil`, then undo is
    not supported.
- `page` - The current UI page.

## Page ##

In Scarlet, we have the concept of a "page." A page is essentially
a model of a single unit of computation. It has a function
that takes the game state and returns a table of Strings to display the player,
a chain of pages to transition to automatically, a table of functions from
game state to String to pages that represent the choices available to the
player and an operation that updates the state .

Everything that the player interacts with is a page: story pages,
basic combat actions (i.e. attack, move, defend), combat options (i.e. attack,
defend, power).

Whether the operation updates the UI or game state depends on the action, for
example a basic combat action would only affect the game state, whereas a combat
option would update the UI state.

A key thing to keep in mind is that pages are _stateless_. They are literally
just a collection of functions that perform various processing on the gamestate,
which is explicitly passed in.

Finally, note that the following are simply the keys that _every_ page is
expected to have. Different types of pages may have additional keys. A key
thing to remember however is that each page _must_ be stateless.

### Body ###

The body is a function that takes the current state and returns a list of
display values, where each entry in the list corresponds to a paragraph. This is
meant to model the content consumed by the user: the next page, a description of
an attack, etc. In the case of a game page, this will likely be a list of
strings. A UI page on the other hand will be a list of tables describing the
(SUIT) widgets added to the UI.

### Automatics ###

Automatics is a function that takes a state, and returns the next event to
transition to automatically, or `nil` if there is no automatic event to
transition to.

This is meant to model transitions to pages that should trigger automatically,
i.e. because of a particular keyword or stat point.

### Choices ###

Choices is a function that takes the current game state, and returns a table
keyed on Strings. The values are the pages to transition to based on the
player's choices.

The table also includes a key `keys` that maps to a list of the keys of the
table. This list defines the order in which the Strings returned by the keys
should be displayed.

This is meant to model explicit choices made by the player, i.e. actions to
take, things to say.

### Transformer ###

The transformer is an operation on the game state, that updates it with
any state changes triggered by this page. The transformer takes the
state to modify, the page we are transitioning from, and the page we
are transitioning to.

Note that the transformer is the ONLY place where side effects are allowed.
Every other component _must_ be proper functions (side-effect free). This
allows us to reexecute the other components as many times as desired (say
after loading a game) without unintended side-effects.

### Execution Order ###

When the game transitions into a page, it performs the following operations:

1. Executes `transformer` on the gamestate
2. Invokes `body`
3. Invokes `automatics`
4. If `automatics` returned a page, go to 8.
5. Get the choice options to display
6. Display the choices to the player
7. Upon receiving player input, map the input to the appropriate choice
function, and use that as a key to get the next page.
8. Transition to the next page.

Note that additional steps may be performed for other page types. This is
simply how we expect the base pages to be operated upon.

For the different types of pages see [Pages](./pages.md).
