--[[
-- Copyright 2017 AKA Russell
--
-- This file is part of Scarlet.
--
-- Scarlet is free software: you can redistribute it and/or modify
-- it under the terms of the GNU General Public License as published by
-- the Free Software Foundation, either version 3 of the License, or
-- (at your option) any later version.
--
-- Scarlet is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty of
-- MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-- GNU General Public License for more details.
--
-- You should have received a copy of the GNU General Public License
-- along with Scarlet.  If not, see <http://www.gnu.org/licenses/>.
--]]
--
--[[
-- Defines UI pages for story pages. The key feature is a function `page`
-- that builds and returns UI story pages. UI story pages define a window into
-- the page body, and provide _keys for scrolling up and down through the page
-- line by line.
--]]
--
local M = {}

local fonts = LOVE and require "ui/fonts"
local load = require 'story/load'
local love = LOVE and require 'love'

local commandpage = require 'story/commands'
local choicebuttons = LOVE and require 'story/choicebuttons'
local pages = require 'pages'
local saveload = require 'utilities/saveloadui'
local scarletfunctions = require 'scarletfunctions'
local scarlettables = require 'scarlettables'
local state = require 'state'
local storyutils = require 'story/storyutils'
local uioptions = require'utilities/optionsui'
local uiutilities = require 'ui/utilities'

 -- We allow commands to take up a third of the screen real estate, and text
 -- to take up a little less than two-thirds
local TEXT_SCREEN_PERCENTAGE = .61

--[[
-- Given a string representing a choice to make, returns the key to use for
-- that choice. If the choice contains a single letter wrapped in parenthesis,
-- then this function returns the Love KeyConstant associated with that letter,
-- or nil if there is no such letter.
--
-- choice: The choice to extract a key for
--
-- return The key wrapped in parenthesis in this choice, or nil if there is
-- no such key
--]]
local function extractkey(choice)
    local startparens = string.find(choice, "%(")
    local endparens = string.find(choice, "%)")
    if startparens and endparens then
        local character = string.sub(choice, startparens + 1, endparens - 1)
        return string.len(character) == 1 and character
    end
end

--[[
-- Given the name of a story page, and the current state of the UI,
-- builds a collection of SUIT widgits that display some of the story.
--
-- In addition to the canonical mappings, the returned operation assumes that
-- the uistate has a mapping `line`, which is the line number of the text
-- currently displayed at the top of the story window.
--
-- Note: This operation also adds the mapping `numlines` to `uistate` that
-- stores the total number of lines of text in this event.
--
-- Recall that a page's transformer is the very *first* thing that gets executed
-- after transitioning to the page. Everything else, including constructing the
-- GUI, gets invoked after.
--
-- uistate: The current state of the game
-- name: The name of the story page body to display
--]]
function M.body(uistate, name)
    local storypage = state.getstory(uistate.core.gamestate.configuration, name)
    M.bodyfrompage(uistate, storypage)
end

function M.bodyfrompage(uistate, storypage)
    local gamestate = uistate.core.gamestate
    local choices = {}
    for i, text in ipairs(storypage.choices(gamestate)._keys) do
        local textwithkey = extractkey(text)
            and text
            or string.format("(%d) %s", i, text)
        table.insert(choices, textwithkey)
    end

    if LOVE then
        storyutils.buildBodyLabel(
            uistate,
            love,
            fonts,
            gamestate.gametext,
            choices,
            TEXT_SCREEN_PERCENTAGE
        )

        uistate.gui.layout:push(2 * love.graphics.getDimensions() / 3, 100)
        storyutils.buildStatisticsPanel(fonts, love, uistate)
        uistate.gui.layout:pop()
    else
        M.consoleBody(uistate, gamestate.gametext, choices)
    end
end

local function displayStatisticsConsole(pc)
    local statistics = pc.statistics
    local lines = {}
    for _, statistic in ipairs(statistics._keys) do
        if statistic == "maxenergy" then
            lines[#lines] = lines[#lines] .. "/" .. statistics[statistic]
        else
            table.insert(lines, statistic .. ": " .. statistics[statistic])
        end
    end
    local statisticsLines = {"------------ Statistics --------------"}
    for i=1,#lines,2 do
        table.insert(
            statisticsLines,
            lines[i] .. "\t\t" .. (i+1 <= #lines and lines[i+1] or "")
        )
    end
    print(table.concat(statisticsLines, "\n"))
end

local function displayAppearanceConsole(pc)
    local lines = {
        "hairstyle: " .. pc.attributes.hairstyle
    }
    for _, slot in ipairs(pc.clothing._keys) do
        table.insert(lines, slot .. ": " .. pc.clothing[slot].name)
    end
    local appearanceLines = {"------------ Wearing --------------"}
    for i=1,#lines,2 do
        table.insert(
            appearanceLines,
            lines[i] .. "\t\t" .. (i < #lines and lines[i+1] or "")
        )
    end
    print(table.concat(appearanceLines, "\n"))
end

local function displayBumstatusConsole(gamestate, pc)
    local bumstatus = storyutils.buildBumText(
        pc.bummarks,
        gamestate.options.bumstatus == "emoji"
    )
    local wrappedbum = storyutils.wrapparagraph(bumstatus, 80)
    print("------------ Bum --------------\n" .. table.concat(wrappedbum, "\n"))
end

--[[
-- Displays the story UI for the console.
--
-- The story UI is just the gametext, wrapped to 80 characters, followed
-- by the player's statistics, and then the available choices.
--
-- uistate - The current state of the game
-- text - A list of strings to display
-- choices- A list of strings representing the choices available to the player
--]]
function M.consoleBody(uistate, text, choices)
    local wrappedtext = storyutils.wraptext(text, 80)
    local paragraphs = {}
    for _, lines in ipairs(wrappedtext) do
        table.insert(paragraphs, table.concat(lines, "\n"))
    end
    if (uistate.command) then
        print(string.rep("*", 80))
    end
    print(table.concat(paragraphs, "\n"))
    local pc = uistate.core.gamestate.characters.pc
    if #choices > 0 then
        if uistate.command == "a" then
            displayStatisticsConsole(pc)
        end
        if uistate.command == "w" then
            displayAppearanceConsole(pc)
        end
        if uistate.command == "b" then
            displayBumstatusConsole(uistate.core.gamestate, pc)
        end
    end
end

local function decrementline(uistate)
    if uistate.textbefore then
        uistate.line = math.max(1, uistate.line - 1)
    end
end

local function decrementpage(uistate)
    if uistate.textbefore then
        uistate.line = math.max(1, uistate.line - uistate.maxlinesonscreen)
    end
end

local function toppage(uistate)
    uistate.line = 1
end

local function incrementline(uistate, maximumlines)
    local topbottom = 1
    if maximumlines > uistate.maxlinesonscreen then
        topbottom = maximumlines - uistate.maxlinesonscreen
    end
    if uistate.textafter then
        uistate.line = math.min(topbottom, uistate.line + 1)
    end
end

local function incrementpage(uistate)
    if uistate.textafter then
        uistate.line = uistate.line + uistate.maxlinesonscreen
    end
end

local function bottompage(uistate, maximumlines)
    local topbottom = 1
    if maximumlines > uistate.maxlinesonscreen then
        topbottom = maximumlines - uistate.maxlinesonscreen
    end
    uistate.line = topbottom
end

local function cleargametext(uipage)
    return pages.page(
        nil,
        function() return uipage end,
        nil,
        function(uistate) uistate.core.gamestate.gametext = {} end
    )
end

--[[
-- Populates uichoices with the story choices the player can make.
--
-- A story choice is a choice that advances the story, rather than a utility
-- choice like save, load, options etc.
--
-- uistate: The current state of the game
-- name: The name of the event we're building choices for
-- uichoices: The ordered table to populate with story choices
-- currentpage: The current page
--]]
local function buildstorychoices(uistate, name, uichoices, currentpage)
    local gamestate = uistate.core.gamestate
    local gamechoices = state
        .getstory(gamestate.configuration, name)
        .choices(gamestate)
    local scarlettables = require 'scarlettables'
    for i, choice in ipairs(gamechoices._keys) do
        local key = extractkey(choice) or tostring(i)
        local choicepage = gamechoices[choice]
        local uichoice = not choicepage.allies
            and M.page(choicepage.name)
            or gamestate.options.combat == "yes"
            and storyutils.combat(uistate, choicepage)
            or M.page(load.skipcombatname(choicepage.name))
        uichoices[key] = cleargametext(uichoice)
        table.insert(uichoices._keys, key)
    end
    if LOVE then
        local componentscreen = require 'characters/componentscreen'
        uichoices["."] = choicebuttons.buttons(uichoices, gamechoices._keys)
        table.insert(uichoices._keys, ".")
        local components = require 'data/components'.buildcomponents()
        local hairstyle
        for _, component in ipairs(components) do
            if component.id == "hairstyle" then
                hairstyle = component
            end
        end
        assert(currentpage)
        if hairstyle then
            local returnpage = pages.page(
                nil,
                function() return currentpage end,
                nil,
                function(uistate1)
                    local pc = uistate1.core.gamestate.characters.pc
                    if uistate.creation and uistate.creation.hairstyle then
                        pc.attributes.hairstyle = uistate.creation.hairstyle
                    end
                    uistate1.creation = nil
                end
            )
            uichoices.f1 = componentscreen.page(hairstyle, returnpage)
            table.insert(uichoices._keys, "f1")
        end
        local pc = uistate.core.gamestate.characters.pc
        local wearables = require 'data/pcitems'
        local count = 2
        local defaultslots = {}
        for name, outfit in pairs(wearables) do
            local firstslot = outfit.slots[1]
            if #outfit.slots == 1 and not defaultslots[firstslot] then
                defaultslots[firstslot] = outfit
            end
        end
        for _, slot in ipairs(pc.clothing._keys) do
            local values = {}
            for name, outfit in pairs(wearables) do
                if outfit.slots[1] == slot then
                    table.insert(values, outfit)
                end
            end 
            local outfitcomponent = {id=slot, label=slot, values=values}
            local key = string.format("f%d", count) 
            count = count + 1
            local returnpage = pages.page(
                nil,
                function() return currentpage end,
                nil,
                function(uistate1)
                    local pc = uistate1.core.gamestate.characters.pc
                    local chosenoutfit = uistate1.creation 
                        and uistate1.creation[slot]
                    if chosenoutfit then
                        local oldoutfit = pc.clothing[slot]
                        for _, slot in ipairs(oldoutfit.slots) do
                            pc.clothing[slot] = defaultslots[slot]
                        end
                        for _, slot in ipairs(wearables[chosenoutfit].slots) do
                            pc.clothing[slot] = wearables[chosenoutfit]
                        end
                    end
                    uistate1.creation = nil
                end
            )
            uichoices[key] = componentscreen.page(outfitcomponent, returnpage)
            table.insert(uichoices._keys, key)
        end
    end
end

--[[
-- Builds and displays the choices available for the
-- current story page in the Console UI.
--
-- uistate - The current state of the game
-- name - The name of the story page to display
--]]
local function displaystorychoicesconsole(uistate, name)
    local gamestate = uistate.core.gamestate
    local gamechoices = state
        .getstory(gamestate.configuration, name)
        .choices(gamestate)
    local display = {string.rep("*", 80)}
    for i, choice in ipairs(gamechoices._keys) do
        local wrappedchoice = storyutils.wraptext({choice}, 80)[1]
        table.insert(
            display, 
            string.format("(%d) %s", i, table.concat(wrappedchoice, "\n"))
        )
    end
    table.insert(display, "")
    table.insert(display, "st(A)tistics\t(W)earing\t(B)um\t\t(T)itle")
    table.insert(display, "(S)ave/(L)oad\t(O)ptions\t(C)haracter")
    print(table.concat(display, "\n"))
end

--[[
-- Builds the map of button ids to pages that represent this page's
-- choices.
--
-- Every manual choice is wrapped in a page that clears out gamestate.gametext.
--
-- uistate - The current state of the UI
--  In addition to the standard mappings, this function assumes the following
--  mappings exist:
--      uistate.line - The line number currently at the top of the screen
--      uistate.maxlinesonpage - The maximum number of lines that can fit on the
--          screen
--      uistate.textbefore - Whether or not there is text before the line at the
--          top of the screen
--      uistate.textafter - Whether or not there is text after the line at the
--          bottom of the screen-
--  These are populated by `buildBodyLabel` (I know, it's a terrible implicit
--  dependency. The current architecture does not make it easy for me to
--  recalculate that information however).
-- name - The name of page to build button ids for
-- storypage - The current page
--]]
local function storychoiceids(uistate, name, storypage)
    local uichoices = {_keys={}}
    buildstorychoices(uistate, name, uichoices, storypage)

    local s = "s"
    uichoices[s] = saveload.saveload()
    table.insert(uichoices._keys, s)

    local l = "l"
    uichoices[l] = saveload.saveload()
    table.insert(uichoices._keys, l)

    local o = "o"
    -- We're wrapping this in a page so that the duplicate story page is
    -- built lazily. In other words, so that we don't enter infinite recursion.
    uichoices[o] = pages.page(
        nil,
        function()
            return uioptions.options(M.page(name))
        end
    )
    table.insert(uichoices._keys, o)
    local c = "c"

    local creationui = require 'characters/creationui'
    uichoices[c] = pages.page(
        nil,
        function()
            return creationui.charactercreationscreen(M.page(name))
        end
    )
    table.insert(uichoices._keys, c)

    local titlescreen = require 'utilities/titlescreen'
    local t = "t"
    uichoices[t] = titlescreen.confirm(function() return M.page(name) end)
    table.insert(uichoices._keys, t)

    if LOVE then
        local up = "up"
        uichoices[up] = M.page(name, decrementline)
        table.insert(uichoices._keys, up)

        local pageup = "pageup"
        uichoices[pageup] = M.page(name, decrementpage)
        table.insert(uichoices._keys, pageup)

        local home = "home"
        uichoices[home] = M.page(name, toppage)
        table.insert(uichoices._keys, home)

        local down = "down"
        uichoices[down] = M.page(
            name,
            function(uistate1)
                incrementline(uistate1, uistate1.numlines)
            end
        )
        table.insert(uichoices._keys, down)

        local pagedown = "pagedown"
        uichoices[pagedown] = M.page(
            name,
            function(uistate1)
                incrementpage(uistate1, uistate1.numlines)
            end
        )
        table.insert(uichoices._keys, pagedown)

        local endkey = "end"
        uichoices[endkey] = M.page(
            name,
            function(uistate1)
                bottompage(uistate1, uistate1.numlines)
            end
        )
        table.insert(uichoices._keys, endkey)

        uichoices["="] = fonts.increasefontsizepage(M.page(name), "bodyfont")
        table.insert(uichoices._keys, "=")
        uichoices["+"] = fonts.increasefontsizepage(
            M.page(name),
            "bodyfont",
            true
        )
        table.insert(uichoices._keys, "+")
        uichoices["-"] = fonts.decreasefontsizepage(M.page(name), "bodyfont")
        table.insert(uichoices._keys, "-")
        local space = "space"
        uichoices[space] = commandpage.commands(uichoices)
        table.insert(uichoices._keys, space)
    end

    return uichoices

end

--[[
-- Returns the transformer for a story page. The transformer is a composition
-- of both the passed in transformer, and the transformer for the specified
-- page. The passed in transformer is applied first.
--
-- This transformer also sets up some state that should be set up at the
-- start of every page, when transitioning to a new page. It sets
-- `uistate.gamestate.storypagename` to this page,
-- and it sets `uistate.line` to 1, which is needed for scrolling.
--
--
-- name: The name of the page to build a UI transformer for
-- transformer: The UI transformer to apply first, if `nil` defaults to the
-- empty operation
--]]
function M.transformer(name, transformer)
    return function(uistate)
        if transformer then
            transformer(uistate)
        end
        local gamestate = uistate.core.gamestate
        local storypage = state.getstory(gamestate.configuration, name)
        if not gamestate.visited[name] then
            storypage.transformer(gamestate)
            gamestate.visited[name] = 1
            for _, paragraph in ipairs(storypage.body(gamestate)) do
                table.insert(gamestate.gametext, paragraph)
            end
            uistate.line = 1
        end
        gamestate.storypagename = name
        uiutilities.clearundo(uistate)
    end
end


-- [[
-- Returns the next UI page to transition to automatically.
--
-- uistate: The current state of the game
-- name: The name of the current event
-- ]]
local function automatics(uistate, name)
    local gamestate = uistate.core.gamestate
    local storypage = state.getstory(gamestate.configuration, name)
    assert(storypage, "No page defined for " .. name)
    local nextpage = storypage.automatics(gamestate)
    if nextpage then
        assert(
            not nextpage.allies,
            "Combat pages cannot be transitioned to automatically."
        )
        return M.page(nextpage.name)
    end
end

--[[
-- Given the name of a story page, builds and returns a UI story page that
-- displays the content of the passed in story page.
--
-- name - The name of the story page to display
-- transformer - A transformer that is composed with the story page's
--      transformer to build a transformer for the UI story page. If nil,
--      defaults to the identity
--]]
function M.page(name, transformer)
    local storypage = pages.page()
    function storypage.automatics(uistate) return automatics(uistate, name) end
    function storypage.choices(uistate)
        return storychoiceids(uistate, name, storypage)
    end
    storypage.transformer = M.transformer(
        name, 
        transformer or scarletfunctions.identity
    )
    storypage.drawgui = function(uistate)
        M.body(uistate, name)
        if LOVE then
            storyutils.buildstorychoices(uistate, name)
        else
            displaystorychoicesconsole(uistate, name)
        end
    end
    return storypage
end

return M
